"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var request = require("request");
var channel_1 = require("./channel");
var Facebook = /** @class */ (function (_super) {
    __extends(Facebook, _super);
    function Facebook(params) {
        return _super.call(this, {
            homedir: params.homedir,
            uid: params.uid,
            platform: 'facebook',
            logger: params.logger
        }) || this;
    }
    Facebook.prototype.sendSenderAction = function (sender, name) {
        return new Promise(function (resolve) {
            var options = {
                uri: 'https://graph.facebook.com/v2.6/me/messages',
                qs: {
                    access_token: process.env.FACEBOOK_PAGE_ACCESS_TOKEN || process.env.RECIME_FACEBOOK_ACCESS_TOKEN
                },
                method: 'POST',
                json: {
                    messaging_type: "RESPONSE",
                    recipient: {
                        id: sender
                    },
                    "sender_action": name
                }
            };
            request(options, function (err, response, body) {
                resolve();
            });
        });
    };
    Facebook.prototype.execute = function (body, parameters, handler) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.params.logger.incoming(body);
            body.entry.forEach(function (pageEntry) {
                var params = {
                    pageEntry: pageEntry,
                    parameters: parameters,
                    handler: handler
                };
                _this.process(params).then(function (data) {
                    resolve(data);
                }, function (err) {
                    reject(err);
                });
            });
        });
    };
    Facebook.prototype.handleRequest = function (ev, parameters, cb) {
        var _this = this;
        var senderID = ev.sender.id;
        return new Promise(function (resolve, reject) {
            _this.sendSenderAction(senderID, 'typing_on').then(function () {
                var send = function (data) {
                    var options = {
                        sender: senderID,
                        data: data,
                        accessToken: process.env.FACEBOOK_PAGE_ACCESS_TOKEN || process.env.RECIME_FACEBOOK_ACCESS_TOKEN
                    };
                    _this.sendMessage(0, options).then(function () {
                        resolve(data);
                    }, function (err) {
                        reject(err);
                    });
                };
                cb(_this.params.uid, ev, _this.params.platform, function (data) {
                    return send(data);
                }, function (err) {
                    send({
                        text: err.message || err
                    });
                });
            });
        });
    };
    Facebook.prototype.sendEachMessage = function (body) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var options = {
                uri: 'https://graph.facebook.com/v2.6/me/messages',
                qs: {
                    access_token: body.accessToken
                },
                method: 'POST',
                json: {
                    messaging_type: "RESPONSE",
                    recipient: {
                        id: body.sender
                    },
                    message: {}
                }
            };
            options.json.message = body.data;
            var data = body.data;
            request(options, function (err, response, result) {
                if (!err && response.statusCode === 200) {
                    _this.params.logger.outgoing(options, response.body);
                    resolve();
                }
                else {
                    var message = {
                        fb: JSON.stringify(result.error),
                        message: response.statusMessage,
                        code: response.statusCode
                    };
                    reject(message);
                }
            });
        });
    };
    Facebook.prototype.sendMessage = function (index, body) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (Array.isArray(body.data)) {
                var data = body.data[index];
                var item = {
                    sender: body.sender,
                    accessToken: body.accessToken,
                    data: data
                };
                if (index < body.data.length) {
                    _this.sendEachMessage(item).then(function () {
                        _this.sendMessage(++index, body).then(function () {
                            resolve();
                        }, function (err) {
                            reject();
                        });
                    }, function (err) {
                        reject();
                    });
                }
                else {
                    resolve();
                }
            }
            else {
                _this.sendEachMessage(body).then(function () {
                    resolve();
                }, function (err) {
                    reject(err);
                });
            }
        });
    };
    Facebook.prototype.process = function (options) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var pageEntry = options.pageEntry;
            var pageID = pageEntry.id;
            var parameters = options.parameters;
            pageEntry.messaging.forEach(function (ev) {
                if (ev.message && !ev.message.is_echo) {
                    _this.handleRequest(ev, parameters, options.handler)
                        .then(resolve)
                        .catch(reject);
                }
                else if (ev.postback) {
                    _this.handleRequest(ev, parameters, options.handler)
                        .then(resolve)
                        .catch(reject);
                }
                else {
                    return resolve();
                }
            });
        });
    };
    Facebook.prototype.verifyToken = function (query) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var token = query['hub.verify_token'];
            if (_this.params.uid === token) {
                resolve(query["hub.challenge"]);
            }
            else {
                reject(403);
            }
        });
    };
    return Facebook;
}(channel_1.Channel));
exports.Facebook = Facebook;
