"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var domain = require("domain");
var request = require("request");
var RecimeLogger = /** @class */ (function () {
    function RecimeLogger(options) {
        this.botId = options.uid;
        this.platform = options.platform;
    }
    RecimeLogger.prototype.incoming = function (body) {
        this.sendRequest({
            direction: "in",
            platform: this.platform,
            res: body
        });
    };
    RecimeLogger.prototype.outgoing = function (reqData, body) {
        this.sendRequest({
            direction: "out",
            platform: this.platform,
            req: reqData,
            res: body
        });
    };
    RecimeLogger.prototype.sendRequest = function (payload) {
        var _this = this;
        return new Promise(function (resolve) {
            request({
                uri: "https://analytics-api.recime.io/v1/bots/" + _this.botId,
                method: 'POST',
                json: true,
                body: payload,
                headers: {
                    authorization: "Token " + process.env.SYSTEM_RECIME_API_KEY,
                    'user-agent': 'recime.io/1.0 bot'
                }
            }, function (err, response, body) {
                if (err) {
                    console.error("Analytics:", err);
                }
                resolve();
            });
        });
    };
    return RecimeLogger;
}());
var ModuleLogger = /** @class */ (function (_super) {
    __extends(ModuleLogger, _super);
    function ModuleLogger(options) {
        var _this = _super.call(this, options.options) || this;
        _this.module = options.module;
        _this.local = domain.create();
        _this.local.on("error", function (err) {
            console.error(err);
        });
        return _this;
    }
    ModuleLogger.prototype.incoming = function (body) {
        var _this = this;
        _super.prototype.incoming.call(this, body);
        this.local.run(function () {
            _this.module.incoming(body);
        });
    };
    ModuleLogger.prototype.outgoing = function (reqData, body) {
        var _this = this;
        _super.prototype.outgoing.call(this, reqData, body);
        this.local.run(function () {
            _this.module.outgoing(reqData, body);
        });
    };
    return ModuleLogger;
}(RecimeLogger));
exports.ModuleLogger = ModuleLogger;
var ConsoleLogger = /** @class */ (function (_super) {
    __extends(ConsoleLogger, _super);
    function ConsoleLogger(options) {
        return _super.call(this, options) || this;
    }
    ConsoleLogger.prototype.incoming = function (body) {
        _super.prototype.incoming.call(this, body);
        console.log("DEBUG: " + JSON.stringify(body));
    };
    ConsoleLogger.prototype.outgoing = function (reqData, body) {
        _super.prototype.outgoing.call(this, reqData, body);
        console.log("DEBUG: " + JSON.stringify(body));
    };
    return ConsoleLogger;
}(RecimeLogger));
exports.ConsoleLogger = ConsoleLogger;
var Logger = /** @class */ (function () {
    function Logger() {
    }
    Logger.init = function (options) {
        return Promise.resolve(new ConsoleLogger(options));
    };
    return Logger;
}());
exports.Logger = Logger;
