"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var objectMapper = require("object-mapper");
var MessageConverter = /** @class */ (function () {
    function MessageConverter(platform) {
        this.platform = platform;
    }
    MessageConverter.prototype.convert = function (data) {
        var platforms = ['telegram', "twilio", 'wechat', 'slack', 'facebook', "viber"];
        if (platforms.indexOf(this.platform) >= 0) {
            var mapped = {
                rawBody: data,
                timestamp: new Date().getTime(),
                event: {}
            };
            var metadata = {
                facebook: {
                    "sender.id": "sender",
                    "message.text": "text",
                    "messsage.nlp": "nlp"
                },
                telegram: {
                    "message.from.id": "sender",
                    "message.text": "text"
                },
                twilio: {
                    "Body": "text",
                    "From": "sender"
                },
                wechat: {
                    "FromUserName": "sender",
                    "Content": "text"
                },
                slack: {
                    "event.user": "sender",
                    "event.text": "text"
                },
                viber: {
                    "sender.id": "sender",
                    "message.text": "text",
                    "event": "event.name"
                }
            };
            var result = objectMapper(data, metadata[this.platform]);
            mapped.text = result.text;
            mapped.sender = result.sender;
            switch (this.platform) {
                case "facebook": {
                    if (data.postback) {
                        mapped.event = {
                            name: data.postback.payload,
                            referral: data.postback.referral
                        };
                    }
                    //location
                    if (data.message && data.message.attachments) {
                        mapped.attachments = data.message.attachments;
                        if (mapped.attachments[0].type === "location") {
                            if (mapped.attachments[0].payload) {
                                var location_1 = mapped.attachments[0].payload.coordinates;
                                mapped.location = {
                                    lat: location_1.lat,
                                    long: location_1.long
                                };
                            }
                        }
                    }
                    if (data.message && data.message.quick_reply) {
                        var quick_reply = data.message.quick_reply;
                        mapped.event = {
                            name: quick_reply.payload,
                        };
                    }
                    break;
                }
                case "viber": {
                    if (data.event !== "message") {
                        mapped.event = {
                            name: data.event.replace(/^conversation_started$/ig, "start")
                        };
                    }
                    // location.
                    if (data.message && data.message.location) {
                        var location_2 = data.message.location;
                        mapped.location = {
                            lat: location_2.lat,
                            long: location_2.lon
                        };
                        delete mapped.text;
                    }
                    // hack for user
                    if (data.user) {
                        mapped.sender = data.user.id;
                    }
                    var regExp = new RegExp(process.env.UID + "-", "ig");
                    if (mapped.text && mapped.text.match(regExp)) {
                        mapped.event = {
                            name: mapped.text.replace(regExp, "")
                        };
                    }
                    break;
                }
                case "telegram": {
                    var regExp = new RegExp("/[A-Za-z0-9]+", "ig");
                    // text
                    if (mapped.text && mapped.text.match(regExp)) {
                        mapped.event = {
                            name: mapped.text.replace(/\//ig, "")
                        };
                    }
                    // process location.
                    if (data.message && data.message.location) {
                        mapped.location = {
                            lat: data.message.location.latitude,
                            long: data.message.location.longitude
                        };
                    }
                    break;
                }
                case "slack": {
                    mapped.event = {
                        name: data.event.type
                    };
                    break;
                }
            }
            return mapped;
        }
        var empty = new Object();
        data.event = data.event || empty;
        data.meta = data.meta || empty;
        data.timestamp = new Date().getTime();
        return data;
    };
    return MessageConverter;
}());
exports.MessageConverter = MessageConverter;
