(function() {
  var Channel, FlakeIdGen, Logger, WeChat, crypto, fs, intformat, request, util,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  fs = require('fs');

  util = require('util');

  crypto = require('crypto');

  FlakeIdGen = require('flake-idgen');

  intformat = require('biguint-format');

  request = require('request');

  Channel = require('./channel').Channel;

  Logger = require('./logger').Logger;

  WeChat = (function(superClass) {
    extend(WeChat, superClass);

    function WeChat(params) {
      WeChat.__super__.constructor.call(this, {
        homedir: params.homedir,
        uid: params.uid,
        platform: 'wechat'
      });
    }

    WeChat.prototype.execute = function(body, parameters, handler) {
      var fromUsername, platform, sendMessage, toUsername, uid;
      uid = this.params.uid;
      platform = this.params.platform;
      fromUsername = body.FromUserName;
      toUsername = body.ToUserName;
      sendMessage = this.sendMessage;
      return new Promise((function(_this) {
        return function(resolve, reject) {
          return Logger.init(_this.params).then(function(logger) {
            var send;
            logger.incoming(body);
            send = function(options) {
              var result;
              result = sendMessage(options.to, options.from, options.data);
              logger.outgoing(options, result);
              return result;
            };
            return handler(uid, body, platform, function(data) {
              var reqData;
              reqData = {
                to: toUsername,
                from: fromUsername,
                data: data
              };
              return resolve(send(reqData));
            }, function(err) {
              var reqData;
              reqData = {
                to: toUsername,
                from: fromUsername,
                data: {
                  text: err.message
                }
              };
              return resolve(send(reqData));
            });
          }, function(err) {
            return console.log(err);
          });
        };
      })(this));
    };

    WeChat.prototype.sendMessage = function(from, to, data) {
      var hash, method, msg, supportedMethods;
      supportedMethods = {
        text: "Content",
        image: "PicUrl",
        link: "Url"
      };
      msg = {};
      msg.FromUserName = from;
      msg.ToUserName = to;
      msg.CreateTime = (new Date).getTime();
      msg.MsgType = {};
      msg.MsgId = intformat(new FlakeIdGen().next());
      msg.MsgType._cdata = 'text';
      msg.Content = {};
      if (!data) {
        return "";
      }
      if (data.text) {
        msg.Content._cdata = data.text;
        delete data.text;
      } else if (data.type && supportedMethods[data.type]) {
        msg.MsgType._cdata = data.type;
        method = supportedMethods[data.type];
        msg[method] = {};
        if (data.content) {
          msg[method]._cdata = data.content;
        } else if (data.picUrl) {
          msg[method]._cdata = data.picUrl;
          hash = crypto.createHash('md5').update(data.picUrl).digest('hex');
          msg.MediaId = {
            _cdata: hash
          };
        } else if (data.url) {
          msg[method]._cdata = data.url;
          if (data.title) {
            msg.Title = {
              _cdata: data.title
            };
          }
          if (data.description) {
            msg.Description = {
              _cdata: data.description
            };
          }
        }
      } else {
        msg.Content._cdata = 'Method not supported.';
      }
      return msg;
    };

    WeChat.prototype.verify = function(query) {
      var accessToken, hash, token;
      accessToken = process.env.WECHAT_ACCESS_TOKEN || process.env.RECIME_WECHAT_ACCESS_TOKEN;
      token = [query['timestamp'], query['nonce'], accessToken].sort().join("");
      hash = crypto.createHash('sha1').update(token).digest('hex');
      return hash === query['signature'];
    };

    return WeChat;

  })(Channel);

  module.exports = WeChat;

}).call(this);
