const util = require("util");
const gulp = require('gulp');

const coffee = require('gulp-coffee');
const nodemon = require('gulp-nodemon');
const sourcemaps = require('gulp-sourcemaps');
const uglify = require('gulp-uglify');
const zip = require('gulp-zip');
const gitignore = require('gulp-gitignore');

const yaml = require('gulp-yaml');
const bump = require('gulp-bump');

const ts = require('gulp-typescript');
const tsProject = ts.createProject('tsconfig.json');

const out = "./dist";
const version = require("moment")().format("YYYY.MM.DDhhmm");

gulp.task ("build-coffee", function(done){
    gulp.src("src/*.coffee")
		.pipe(coffee())
		.pipe(gitignore())
	  	.pipe(gulp.dest(out))
	.on("end", ()=>done());
});

gulp.task ("build-ts", function(done){
    gulp.src(["src/**/*.ts", "src/engine/*.ts"])
		.pipe(tsProject())
		.pipe(gitignore())
	  	.pipe(gulp.dest(out))
	.on("end", ()=>done());
});

gulp.task("bump", function(done){
	gulp.src("package.json")
		.pipe(bump({version : version}))
		.pipe(gulp.dest("."))
	.on("end", ()=>done());
});

gulp.task('build', ['build-coffee', 'build-ts', "bump"]);

gulp.task("default", ["build"], ()=>{
	var started = false;
	return nodemon({
		script: 'app.js'
    , ext : "coffee, ts"
    , tasks : ['build-coffee', 'build-ts']
	}).on('start', function () {
		// to avoid nodemon being started multiple times
		// thanks @matthisk
		if (!started) {
			cb();
			started = true;
		}
	});
});
