import fs = require('fs');
import path = require('path');
import domain = require("domain");
import request = require('request');
import vm = require('vm');

import resolveUrl = require("resolve-url");

import {MainModule} from "./main-module";

class RecimeError extends Error {
    constructor(message){
        super(message);
    }
};

export class Runnable{
    private options: any;

    constructor(options){
        this.options = options;
    }

    run(data) : Promise<any> {
         return new Promise((resolve, reject)=>{
            const options = this.options;
            const mainModule = new MainModule(options);

            fs.readFile(mainModule.path(), 'utf-8', (err, code)=>{
                if (err){
                    return reject(err);
                }

                // uid
                options.env['UID'] = options.uid;
                options.env['HOME_DIR'] = options.homedir;
                // pass in platform for external modules to take advantage of.
                options.env['PLATFORM'] = options.platform || "web";
                // aws region
                options.env['REGION'] = process.env.AWS_REGION;

                const p = {
                    env : options.env
                }

                let sandbox = {
                    console : console,
                    exports : exports,
                    process : p,
                    require : (module:string)=>{
                       return require(mainModule.normalizedDepsPath(module));
                    },
                    url: (p)=>{
                        if (process.env.AWS_LAMBDA_FUNCTION_NAME){
                           const asset = p.replace(/(\/)?assets\//ig, "")
                           return `https://assets.recime.io/${path.normalize(`${options.uid}/${asset}`)}`
                        }
                        return `https://${options.host}/${path.normalize(`${options.uid}/v1/${p}`)}`;
                     },
                    __dirname: mainModule.dir
                };

                const Module = vm.runInNewContext(code, sandbox);

                if (typeof Module === 'undefined'){
                    reject(new RecimeError("Could not resolve the main module."));
                    return;
                }     
                const local = domain.create();
                
                local.on("error", (err)=>{
                    reject(new RecimeError(err.message));
                });

                local.run(function () {
                    // export default
                    const Main = Module.Bot || Module;
                    const isInstanceObj = !!Main.prototype && !!Main.prototype.constructor.name;

                    if (isInstanceObj){   
                        const main = new Main(data);
                    
                        if (Main.constructor.length === 0)
                            return reject(new RecimeError("Invalid Bot class definition."));

                        switch(main.constructor.name){
                            case "Bot":{
                                var ret = main.execute(resolve);
                                if (ret && ret.then && main.execute.length === 0) {
                                    ret.then(resolve, function (err) {
                                        reject(new RecimeError(err.message));
                                    });
                                }
                                break;
                            }
                            default :{
                                reject(new RecimeError(`Invalid class name ${main.constructor.name}`));
                            }
                        }
                    } else {
                        return Module(data, resolve);
                    }
                });
            });
         });
         
    }
}
