import domain = require("domain");
import path = require('path');
import request = require("request");

import {MainModule} from "./engine/main-module";

interface Module {
    incoming(body: object);
    outgoing(reqData, body:object);
}

class RecimeLogger implements Module {
    private botId: any;
    private platform: any;

    constructor(options){
        this.botId = options.uid;
        this.platform = options.platform;
    }

    incoming(body: any){
        this.sendRequest({
            direction : "in",
            platform: this.platform,
            res : body
        });
    }

    outgoing(reqData, body){
        this.sendRequest({
            direction : "out",
            platform: this.platform,
            req : reqData,
            res : body
        });
    }

    sendRequest(payload) {
        return new Promise((resolve) => {
            request({
                uri: `https://analytics-api.recime.io/v1/bots/${this.botId}`,
                method: 'POST',
                json: true,
                body: payload,
                headers: {
                    authorization: `Token ${process.env.SYSTEM_RECIME_API_KEY}`,
                    'user-agent': 'recime.io/1.0 bot'
                }
            }, (err, response, body) => {
                if (err) {
                    console.error("Analytics:", err);
                }
                resolve();
            });
        });
    }
}

export class ModuleLogger extends RecimeLogger {
    private module: any;
    private local: any;

    constructor(options){
        super(options.options);
        this.module = options.module;
        this.local = domain.create();
        this.local.on("error", (err)=>{
            console.error(err);
        });
    }
    
    incoming(body:any){
        super.incoming(body);

        this.local.run(()=>{
            this.module.incoming(body);
        });
    }
    
    outgoing(reqData, body:any){
        super.outgoing(reqData, body);

        this.local.run(()=>{
            this.module.outgoing(reqData, body);
        });
    }
}

export class ConsoleLogger extends RecimeLogger {
    constructor(options){
        super(options);
    }

    incoming(body:object){
        super.incoming(body);
        console.log(`DEBUG: ${JSON.stringify(body)}`);
    }
    outgoing(reqData, body:object){
        super.outgoing(reqData, body);
        console.log(`DEBUG: ${JSON.stringify(body)}`);
    }
} 

export class Logger {
    static init(options) : Promise<Module> {
        return Promise.resolve(new ConsoleLogger(options))
    } 
}
