import objectMapper = require("object-mapper");
import {MsgObject} from "./msg-object";

interface Location{
    lat : number,
    long: number
}

interface Message {
    text? : string,
    sender? : string,
    event? : any,
    timestamp: number,
    location?: Location,
    attachments?: any,
    nlp?: object,
    rawBody : any
}

export class MessageConverter {
    private platform: any;

    constructor(platform){
        this.platform = platform;
    }
    
    convert(data: any){
        const platforms = ['telegram', "twilio" , 'wechat', 'slack', 'facebook', "viber"];
        if (platforms.indexOf(this.platform) >= 0){
            const mapped: Message = {
                rawBody:  data,
                timestamp : new Date().getTime(),
                event : {}
            };

            const metadata = {
                facebook : {
                    "sender.id" : "sender",
                    "message.text": "text",
                    "messsage.nlp": "nlp"
                },
                telegram : {
                    "message.from.id" : "sender",
                    "message.text": "text"
                },
                twilio : {
                    "Body"  : "text",
                    "From": "sender"
                },
                wechat : {
                    "FromUserName" : "sender",
                    "Content": "text"
                },
                slack : {
                    "event.user" : "sender",
                    "event.text": "text"
                },
                viber: {
                    "sender.id": "sender",
                    "message.text": "text",
                    "event": "event.name"
                }
            };

            const result = objectMapper(data, metadata[this.platform]);

            mapped.text = result.text;
            mapped.sender = result.sender;

            switch(this.platform){
                case "facebook":{
                    if (data.postback){
                        mapped.event = {
                            name : data.postback.payload,
                            referral : data.postback.referral
                        }
                    }
                    //location
                    if (data.message && data.message.attachments){
                        mapped.attachments = data.message.attachments;
                        if (mapped.attachments[0].type === "location"){
                            if (mapped.attachments[0].payload){
                                const location = mapped.attachments[0].payload.coordinates;
                                mapped.location = {
                                lat : location.lat,
                                long: location.long
                                }
                            }
                        }
                    }
                    if (data.message && data.message.quick_reply){
                        const quick_reply = data.message.quick_reply;
                        mapped.event = {
                            name : quick_reply.payload,
                        };
                    }

                    break;
                }
                case "viber":{
                    if (data.event !== "message"){
                        mapped.event = {
                            name : data.event.replace(/^conversation_started$/ig, "start")
                        };
                    }
                    // location.
                    if (data.message && data.message.location) {
                        const location = data.message.location;
                        mapped.location = {
                            lat: location.lat,
                            long: location.lon
                        };
                        delete mapped.text;
                    }
                    
                    // hack for user
                    if (data.user){
                        mapped.sender = data.user.id;
                    }

                    const regExp = new RegExp(`${process.env.UID}-`, "ig")

                    if (mapped.text && mapped.text.match(regExp)){
                        mapped.event ={
                            name :  mapped.text.replace(regExp, "")
                        }
                    }
                    break;
                }
                case "telegram":{
                    const regExp = new RegExp("/[A-Za-z0-9]+", "ig");
                    // text
                    if (mapped.text && mapped.text.match(regExp)) {
                        mapped.event = {
                            name: mapped.text.replace(/\//ig, "")
                        };
                    }
                    // process location.
                    if (data.message && data.message.location){
                         mapped.location = {
                            lat: data.message.location.latitude,
                            long: data.message.location.longitude
                        };
                    }
                    break;
                }
                case "slack" :{
                    mapped.event = {
                        name : data.event.type
                    };
                    break;
                }
            }

            return mapped; 
        }

        const empty = new Object();
        
        data.event = data.event || empty;
        data.meta = data.meta || empty;
        data.timestamp = new Date().getTime();

        return data;
    }
}