import request = require("request");

import {Channel} from './channel';
import {Logger} from "./logger";

export class Viber extends Channel {
    private logger: any;
   
    constructor(params: any){
        super({
            homedir : params.homedir,
            uid : params.uid,
            platform : 'viber'
        });
    }
    accountInfo(){
        return new Promise((resolve)=>{
            const url = `https://chatapi.viber.com/pa/get_account_info`;
            const options = {
                json : true,
                url : url,
                headers : {
                    "X-Viber-Auth-Token" : process.env.RECIME_VIBER_ACCESS_TOKEN
                },
                body : {}
            };
            request(options, (err, response, result)=>{
                resolve(result);
            });
        });
    }

    execute(body, parameters, handler){
        return new Promise ((resolve)=>{
            this.accountInfo().then((user)=>{
                Logger.init(this.params).then((logger)=>{
                    logger.incoming(body);

                    handler(this.params.uid, body, "viber", (result)=>{
                        // force to array. 
                        result = result.length ? result : [result];    
                        
                        this.sendMessage(0, user, body, result, logger).then(()=>{
                            resolve()
                        });
                    });
                });
            });
        });
    }

    sendMessage(index, user, body, result:any[], logger){
        const method = "send_message";
        return new Promise((resolve, reject)=>{
            const message = result[index];
            const sender = body.sender || body.user;
                
            message.receiver = sender.id;
            message.tracking_data = this.params.uid;
            message.sender = {
                name : user.name,
                avatar : `https://icons.recime.io/${this.params.uid}`
            };
            message.type = message.type || "text";
        
            const url = `https://chatapi.viber.com/pa/${method}`;
            const options = {
                json : true,
                url : url,
                headers : {
                    "X-Viber-Auth-Token" : process.env.RECIME_VIBER_ACCESS_TOKEN
                },
                body : message
            };
            request(options, (err, response, r)=>{
                logger.outgoing(options, r);
                
                if (index + 1 < result.length){
                    this.sendMessage(++index, user, body, result, logger).then(resolve);
                } else {
                    return resolve();
                }  
            });
        });
    }
}