fs = require('fs')
util = require('util')
crypto = require('crypto')

FlakeIdGen = require('flake-idgen')

intformat = require('biguint-format')
request = require('request')

Channel = require('./channel').Channel

Logger = require('./logger').Logger

class WeChat extends Channel
    constructor:(params)->
        super
            homedir : params.homedir
            uid : params.uid
            platform : 'wechat'

    execute:(body, parameters, handler)->
        uid = @params.uid
        platform = @params.platform
        fromUsername = body.FromUserName
        toUsername = body.ToUserName
        sendMessage = @sendMessage
       
        new Promise (resolve, reject)=>
            Logger.init(@params).then (logger)=>
                logger.incoming(body)

                send = (options)=>
                     result = sendMessage(options.to, options.from, options.data)
                     logger.outgoing(options, result)
                     result

                handler uid, body, platform, (data)->
                    reqData = 
                        to : toUsername
                        from : fromUsername
                        data : data
                    resolve(send(reqData))
                , (err)->
                    reqData = 
                        to : toUsername
                        from : fromUsername
                        data :
                            text : err.message
                    resolve(send(reqData))
            ,(err)=>
                console.log(err)
                  
    sendMessage:(from, to, data)->
        supportedMethods =
            text  : "Content"
            image : "PicUrl"
            link  : "Url"
        msg = {}
        msg.FromUserName = from
        msg.ToUserName = to 
        # the value of time as an integer number of seconds since the Epoch.
        msg.CreateTime = (new Date).getTime()
        msg.MsgType = {}
        msg.MsgId = intformat(new FlakeIdGen().next())
        msg.MsgType._cdata = 'text'
        msg.Content = {}

        if !data
            return ""

        if data.text
            msg.Content._cdata = data.text
            delete data.text
        else if data.type and supportedMethods[data.type]
            msg.MsgType._cdata = data.type
            method =  supportedMethods[data.type]
            msg[method] = {}
            
            if data.content
                msg[method]._cdata = data.content
            else if data.picUrl
                 msg[method]._cdata =  data.picUrl
                 hash = crypto.createHash('md5').update(data.picUrl).digest('hex');
                 msg.MediaId = 
                    _cdata : hash
            else if data.url
                msg[method]._cdata = data.url
                if data.title
                    msg.Title =
                        _cdata : data.title
                if data.description
                    msg.Description =
                        _cdata : data.description
        else
            msg.Content._cdata = 'Method not supported.'

        msg

    verify:(query)->
        accessToken = process.env.WECHAT_ACCESS_TOKEN || process.env.RECIME_WECHAT_ACCESS_TOKEN
        token = [query['timestamp'], query['nonce'], accessToken].sort().join("")
        hash = crypto.createHash('sha1').update(token).digest('hex')
        
        hash is query['signature']


module.exports = WeChat
