const assert = require('assert');
const MessageConverter = require(__dirname + '/../message-converter').MessageConverter;
const XML = require(__dirname + '/../xml').XML


describe("converter", ()=>{
    it("should assert facebook", ()=>{

        const converter = new MessageConverter("facebook");

        const data = {
            "sender": {
                "id":"1223182967789737"
            },
            "recipient":{
                "id":"769372433211496"
            },
            "timestamp":1494892213769,
            "message":{
                "mid":"mid.$cAAK7vV_yB-5iP8o-CVcDoT1gIade",
                "seq":11504,
                "text":"who are you?"
            }
        };

        const result =  converter.convert(data);

        assert.equal("who are you?", result.text);
        assert.equal("1223182967789737", result.sender);
        assert.ok(result.rawBody);
    });

    it("should assert telegram", ()=>{
        const converter = new MessageConverter("telegram");

        const data = {
          update_id: 858720751,
          message: {
            message_id: 430,
            from: {
              id: 159538549,
              last_name: "Doe",
              first_name: "John",
              username: "john"
            },
            chat: {
              id: 159538549,
              last_name: "Doe",
              first_name: "Jane",
              username: "jane",
              type: "private"
            },
            date: (new Date).getTime(),
            text: "Hello world!"
          }
        };
        const result =  converter.convert(data);

        assert.equal("Hello world!", result.text);
        assert.equal("159538549", result.sender);
        assert.ok(result.rawBody);
    });

    it ("should assert wechat", ()=>{
        const xml = "<xml><ToUserName><![CDATA[toUser]]></ToUserName><FromUserName><![CDATA[fromUser]]></FromUserName> <CreateTime>1348831860</CreateTime><MsgType><![CDATA[text]]></MsgType><Content><![CDATA[this is a test]]></Content><MsgId>1234567890123456</MsgId></xml>";
         XML.ifConvert(xml).then((data)=>{
            const converter = new MessageConverter("wechat");
            const result =  converter.convert(data);

            assert.equal("this is a test", result.text);
            assert.equal("fromUser", result.sender);
            assert.ok(result.rawBody);
         }, (err)=>{
                throw err
         });
    });

    it ("should assert slack", ()=>{
         const data = {
          token: 'enBr8Mh1BdVeoGp2FvCxCDl8',
          team_id: 'T257411PU',
          api_app_id: 'A580074JH',
          event: {
            "type": "message",
            "subtype": "bot_message",
            "ts": "1358877455.000010",
            "text": "Pushing is the answer",
            "bot_id": "BB12033",
            "username": "github",
            "icons": {}
          },
          type: 'event_callback',
          authed_users: ['U57UZ7REY'],
          event_id: 'Ev577XSNFJ',
          event_time: 1493800160
        };
        const converter = new MessageConverter("slack");
        const result =  converter.convert(data);

        assert.equal("Pushing is the answer", result.text);
        assert.equal("github", result.sender);
        assert.ok(result.rawBody);

    });
});
