(function() {
  var SchemaValidator, assert;

  assert = require('assert');

  SchemaValidator = require(__dirname + '/../schema-validator').SchemaValidator;

  describe('slack', function() {
    return describe('validate()', function() {
      return it('should return true for valid', function() {
        var data;
        data = {
          type: "url_verification",
          token: "enBr8Mh1BdVeoGp2FvCxCDl8",
          challenge: "9HaRb9H2dEHT6NGiZFoL6GWQ01qBOJV4XuSkv6U6P3ZL92SznTR6"
        };
        return SchemaValidator.validate(data, 'slack').then(function(result) {
          return assert.equal(result, true);
        }, function(err) {
          return assert.ok(false);
        });
      });
    });
  });

  describe('slack-event', function() {
    return describe('validate()', function() {
      return it('should return true for valid', function() {
        var data;
        data = {
          token: 'enBr8Mh1BdVeoGp2FvCxCDl8',
          team_id: 'T257411PU',
          api_app_id: 'A580074JH',
          event: {
            type: 'member_joined_channel',
            user: 'U57UZ7REY',
            channel: 'C57TKS2BU',
            channel_type: 'C',
            event_ts: '1493800160.199104'
          },
          type: 'event_callback',
          authed_users: ['U57UZ7REY'],
          event_id: 'Ev577XSNFJ',
          event_time: 1493800160
        };
        return SchemaValidator.validate(data, 'slack-event').then(function(result) {
          return assert.equal(result, true);
        }, function(err) {
          return assert.ok(false);
        });
      });
    });
  });

  describe('wechat', function() {
    return describe('validate()', function() {
      return it('should return true for valid', function() {
        var data;
        data = {
          ToUserName: 'to',
          FromUserName: 'from',
          CreateTime: '111333222',
          MsgType: 'text',
          Content: 'Hello World!',
          MsgId: '11111'
        };
        return SchemaValidator.validate(data, 'wechat').then(function(result) {
          return assert.equal(result, true);
        }, function(err) {
          return assert.ok(false);
        });
      });
    });
  });

  describe('telegram', function() {
    return describe('validate()', function() {
      var platform;
      platform = 'telegram';
      it('should return true for valid', function() {
        var data;
        data = {
          update_id: 858720751,
          message: {
            message_id: 430,
            from: {
              id: 159538549,
              last_name: "Doe",
              first_name: "John",
              username: "john"
            },
            chat: {
              id: 159538549,
              last_name: "Doe",
              first_name: "Jane",
              username: "jane",
              type: "private"
            },
            date: (new Date).getTime(),
            text: "Hello world!"
          }
        };
        return SchemaValidator.validate(data, platform).then(function(result) {
          return assert.equal(result, true);
        }, function(err) {
          throw err;
        });
      });
      return it('should return false for invalid', function() {
        var data;
        data = {
          update_id: 1111,
          message: {
            text: "Hello world!"
          }
        };
        return SchemaValidator.validate(data, platform).then(function(result) {
          return assert.equal(result, false);
        }, function(err) {
          throw err;
        });
      });
    });
  });

  describe('schema', function() {
    return describe('findOne()', function() {
      return it('should assert a platform', function() {
        var data;
        data = {
          ToUserName: 'to',
          FromUserName: 'from',
          CreateTime: '111333222',
          MsgType: 'text',
          Content: 'Hello World!',
          MsgId: '11111'
        };
        return SchemaValidator.findOne(data).then(function(platform) {
          return assert.equal(platform, 'wechat');
        }, function(err) {
          throw err;
        });
      });
    });
  });

}).call(this);
